@@ -1,9 +1,10 @@
-__version_info__ = (0, 4, 1, 2)
-__version__ = ".".join("%s" % v for v in __version_info__)
+__version_info__    = (0, 5, 0, 2)
+__version__         = ".".join("%s" % v for v in __version_info__)
 
-__author__ = 'Rhett Garber <rhettg@gmail.com>'
+__author__          = 'Yelp <yelplabs@yelp.com>'
 
-__credits__ = [
+__credits__         = [
+    'Rhett Garber <rhettg@gmail.com>',
     'Eugene Baumstein <eugene.baumstein@gmail>',
     'James Brown <jbrown@yelp.com>',
     'Adam Derewecki <derewecki@gmail.com>',
