@@ -1,21 +1,41 @@
+"""
+Tron schedulers
+
+ A scheduler has a simple interface.
+
+ class Scheduler(object):
+
+    schedule_on_complete = <bool>
+
+    def next_run_time(self, last_run_time):
+        <returns datetime>
+
+
+ next_run_time() should return a datetime which is the time the next job run
+ will be run.
+
+ schedule_on_complete is a bool that identifies if this scheduler should have
+ jobs scheduled with the start_time of the previous run (False), or the
+ end time of the previous run (False).
+"""
 import logging
 
 from pytz import AmbiguousTimeError, NonExistentTimeError
-from tron.config import config_parse, schedule_parse
+from tron.config import schedule_parse
 
 from tron.utils import groctimespecification
 from tron.utils import timeutils
 
 
-log = logging.getLogger('tron.scheduler')
+log = logging.getLogger(__name__)
 
 
 def scheduler_from_config(config, time_zone):
     """A factory for creating a scheduler from a configuration object."""
-    if isinstance(config, config_parse.ConfigConstantScheduler):
+    if isinstance(config, schedule_parse.ConfigConstantScheduler):
         return ConstantScheduler()
 
-    if isinstance(config, config_parse.ConfigIntervalScheduler):
+    if isinstance(config, schedule_parse.ConfigIntervalScheduler):
         return IntervalScheduler(interval=config.timedelta)
 
     if isinstance(config, schedule_parse.ConfigDailyScheduler):
@@ -30,23 +50,11 @@ def scheduler_from_config(config, time_zone):
 
 
 class ConstantScheduler(object):
-    """The constant scheduler schedules the first job run. The next job run
-    is scheduled when this first run is finished.
-    """
+    """The constant scheduler schedules a new job immediately."""
+    schedule_on_complete = True
 
-    def __init__(self, *args, **kwargs):
-        super(ConstantScheduler, self).__init__(*args, **kwargs)
-        self.time_zone = None
-
-    def next_runs(self, job):
-        if job.next_to_finish():
-            return []
-
-        job_runs = job.build_runs()
-        for job_run in job_runs:
-            job_run.set_run_time(timeutils.current_time())
-
-        return job_runs
+    def next_run_time(self, _):
+        return timeutils.current_time()
 
     def __str__(self):
         return "CONSTANT"
@@ -62,10 +70,11 @@ class DailyScheduler(object):
     """Wrapper around SpecificTimeSpecification in the Google App Engine cron
     library
     """
+    schedule_on_complete = False
 
     def __init__(self, ordinals=None, weekdays=None, months=None,
                  monthdays=None, timestr=None, time_zone=None,
-                 start_time=None, string_repr=None):
+                 string_repr=None):
         """Parameters:
           timestr     - the time of day to run, as 'HH:MM'
           ordinals    - first, second, third &c, as a set of integers in 1..5 to
@@ -78,19 +87,17 @@ class DailyScheduler(object):
           timezone    - the optional timezone as a string for this specification.
                         Defaults to UTC - valid entries are things like
                         Australia/Victoria or PST8PDT.
-          start_time  - Backward-compatible parameter for DailyScheduler
           string_repr - Original string representation this was parsed from,
                         if applicable
         """
-        self.ordinals = ordinals
-        self.weekdays = weekdays
-        self.months = months
-        self.monthdays = monthdays
-        self.timestr = timestr or '00:00'
-        self.time_zone = time_zone
-        self.string_repr = string_repr
-
-        self._time_spec = None
+        self.ordinals               = ordinals
+        self.weekdays               = weekdays
+        self.months                 = months
+        self.monthdays              = monthdays
+        self.timestr                = timestr or '00:00'
+        self.time_zone              = time_zone
+        self.string_repr            = string_repr
+        self._time_spec             = None
 
     @property
     def time_spec(self):
@@ -107,37 +114,27 @@ class DailyScheduler(object):
                 months=self.months,
                 monthdays=self.monthdays,
                 timestr=self.timestr,
-                timezone=self.time_zone.zone if self.time_zone else None)
+                timezone=self.time_zone.zone if self.time_zone else None,
+            )
         return self._time_spec
 
-    def next_runs(self, job):
-        # Find the next time to run
-        if job.runs:
-            start_time = job.runs[0].run_time
-        else:
+    def next_run_time(self, start_time):
+        """Find the next time to run."""
+        if not start_time:
             start_time = timeutils.current_time()
-            if self.time_zone:
-                try:
-                    start_time = self.time_zone.localize(start_time,
-                                                         is_dst=None)
-                except AmbiguousTimeError:
-                    # We are in the infamous 1 AM block which happens twice on
-                    # fall-back. Pretend like it's the first time, every time.
-                    start_time = self.time_zone.localize(start_time,
-                                                         is_dst=True)
-                except NonExistentTimeError:
-                    # We are in the infamous 2:xx AM block which does not
-                    # exist. Pretend like it's the later time, every time.
-                    start_time = self.time_zone.localize(start_time,
-                                                         is_dst=True)
+        elif self.time_zone:
+            try:
+                start_time = self.time_zone.localize(start_time, is_dst=None)
+            except AmbiguousTimeError:
+                # We are in the infamous 1 AM block which happens twice on
+                # fall-back. Pretend like it's the first time, every time.
+                start_time = self.time_zone.localize(start_time, is_dst=True)
+            except NonExistentTimeError:
+                # We are in the infamous 2:xx AM block which does not
+                # exist. Pretend like it's the later time, every time.
+                start_time = self.time_zone.localize(start_time, is_dst=True)
 
-        run_time = self.time_spec.GetMatch(start_time)
-
-        job_runs = job.build_runs()
-        for job_run in job_runs:
-            job_run.set_run_time(run_time)
-
-        return job_runs
+        return self.time_spec.GetMatch(start_time)
 
     def __str__(self):
         # Backward compatible string representation which also happens to be
@@ -165,19 +162,14 @@ class IntervalScheduler(object):
     """The interval scheduler runs a job (to success) based on a configured
     interval.
     """
+    schedule_on_complete = False
 
     def __init__(self, interval=None):
         self.interval = interval
-        self.time_zone = None
 
-    def next_runs(self, job):
-        run_time = timeutils.current_time() + self.interval
-
-        job_runs = job.build_runs()
-        for job_run in job_runs:
-            job_run.set_run_time(run_time)
-
-        return job_runs
+    def next_run_time(self, last_run_time):
+        last_run_time = last_run_time or timeutils.current_time()
+        return last_run_time + self.interval
 
     def __str__(self):
         return "INTERVAL:%s" % self.interval
